package apfel;

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Choice;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.Panel;
import java.awt.TextField;
import java.awt.Window;

import org.omg.CORBA.ORB;

import apfel.calc.ApfelCalculator;
import apfel.calc.ApfelCalculatorException;
import apfel.calc.ApfelCalculatorFactory;

public class ApfelFrame extends Frame {

  private static final long      serialVersionUID = 1L;

  private ORB                    orb              = null;

  private ApfelCalculatorFactory factory          = null;

  private ApfelPanel             apfelPanel       = null;

  private Panel                  controlPanel     = null;

  private Panel                  panel1           = null;

  private Label                  label            = null;

  private TextField              textX            = null;

  private Panel                  panel2           = null;

  private Panel                  panel            = null;

  private Panel                  panel3           = null;

  private Label                  label1           = null;

  private TextField              textY            = null;

  private Label                  label2           = null;

  private TextField              textZoom         = null;

  private Label                  label3           = null;

  private TextField              textPixel        = null;

  private Panel                  panel4           = null;

  private Label                  label4           = null;

  private TextField              textIterations   = null;

  private Button                 buttonReset      = null;

  private Panel                  panel5           = null;

  private Panel                  panel6           = null;

  private Panel                  panel7           = null;

  private Label                  label5           = null;

  private TextField              textTime         = null;

  private Choice                 choiceCalc       = null;

  private Button                 buttonRefresh    = null;

  /**
   * This method initializes panel
   * 
   * @return java.awt.Panel
   */
  private Panel getApfelPanel() {
    if (apfelPanel == null) {
      apfelPanel = new ApfelPanel();
      apfelPanel.setLayout(null);
      apfelPanel.setSize(new java.awt.Dimension(400, 400));
      apfelPanel.setParent(this);
    }
    return apfelPanel;
  }

  /**
   * This is the default constructor
   */
  public ApfelFrame(ORB orb) {
    super();

    this.orb = orb;
    initialize();
  }

  /**
   * This method initializes this
   * 
   * @return void
   */
  private void initialize() {
    factory = new ApfelCalculatorFactory(orb);

    this.setLayout(new BorderLayout());
    this.setSize(400, 550);
    this.setTitle("SAR - Course Middleware - Lab 3");
    this.add(getApfelPanel(), java.awt.BorderLayout.NORTH);
    this.add(getControlPanel(), java.awt.BorderLayout.CENTER);
    this.addWindowListener(new java.awt.event.WindowAdapter() {
      public void windowClosed(java.awt.event.WindowEvent e) {
        System.exit(0);
      }

      public void windowClosing(java.awt.event.WindowEvent e) {
        Window theWindow = e.getWindow();
        theWindow.dispose();
      }
    });

    onX0Changed();
    onY0Changed();
    onZoomChanged();
    onIterationsChanged();
    onUnitChanged();
    onTimeChanged();
  }

  /**
   * This method initializes controlPanel
   * 
   * @return java.awt.Panel
   */
  private Panel getControlPanel() {
    if (controlPanel == null) {
      GridLayout gridLayout1 = new GridLayout();
      gridLayout1.setRows(4);
      gridLayout1.setColumns(2);
      controlPanel = new Panel();
      controlPanel.setLayout(gridLayout1);
      controlPanel.add(getPanel1(), null);
      controlPanel.add(getPanel(), null);
      controlPanel.add(getPanel2(), null);
      controlPanel.add(getPanel3(), null);
      controlPanel.add(getPanel4(), null);
      controlPanel.add(getPanel5(), null);
      controlPanel.add(getPanel7(), null);
      controlPanel.add(getPanel6(), null);
    }
    return controlPanel;
  }

  /**
   * This method initializes panel1
   * 
   * @return java.awt.Panel
   */
  private Panel getPanel1() {
    if (panel1 == null) {
      label = new Label();
      label.setText("X");
      panel1 = new Panel();
      panel1.add(label, null);
      panel1.add(getTextX(), null);
    }
    return panel1;
  }

  /**
   * This method initializes textField
   * 
   * @return java.awt.TextField
   */
  private TextField getTextX() {
    if (textX == null) {
      textX = new TextField();
      textX.setColumns(15);
      textX.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(java.awt.event.ActionEvent e) {
          try {
            double x = Double.parseDouble(textX.getText());
            apfelPanel.setX0(x);
            apfelPanel.repaint();
          } catch (NumberFormatException ex) {
          }
        }
      });
    }
    return textX;
  }

  /**
   * This method initializes panel2
   * 
   * @return java.awt.Panel
   */
  private Panel getPanel2() {
    if (panel2 == null) {
      label1 = new Label();
      label1.setText("Y");
      panel2 = new Panel();
      panel2.add(label1, null);
      panel2.add(getTextY(), null);
    }
    return panel2;
  }

  /**
   * This method initializes panel
   * 
   * @return java.awt.Panel
   */
  private Panel getPanel() {
    if (panel == null) {
      label2 = new Label();
      label2.setText("Zoom");
      panel = new Panel();
      panel.add(label2, null);
      panel.add(getTextZoom(), null);
    }
    return panel;
  }

  /**
   * This method initializes panel3
   * 
   * @return java.awt.Panel
   */
  private Panel getPanel3() {
    if (panel3 == null) {
      label3 = new Label();
      label3.setText("Pixel");
      panel3 = new Panel();
      panel3.add(label3, null);
      panel3.add(getTextPixel(), null);
    }
    return panel3;
  }

  /**
   * This method initializes textField1
   * 
   * @return java.awt.TextField
   */
  private TextField getTextY() {
    if (textY == null) {
      textY = new TextField();
      textY.setColumns(15);
      textY.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(java.awt.event.ActionEvent e) {
          try {
            double y = Double.parseDouble(textY.getText());
            apfelPanel.setY0(y);
            apfelPanel.repaint();
          } catch (NumberFormatException ex) {
          }
        }
      });
    }
    return textY;
  }

  /**
   * This method initializes textField2
   * 
   * @return java.awt.TextField
   */
  private TextField getTextZoom() {
    if (textZoom == null) {
      textZoom = new TextField();
      textZoom.setColumns(12);
      textZoom.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(java.awt.event.ActionEvent e) {
          try {
            long zoom = Long.parseLong(textZoom.getText());
            apfelPanel.setZoomFactor(zoom);
            apfelPanel.repaint();
          } catch (NumberFormatException ex) {
          }
        }
      });
    }
    return textZoom;
  }

  /**
   * This method initializes textPixel
   * 
   * @return java.awt.TextField
   */
  private TextField getTextPixel() {
    if (textPixel == null) {
      textPixel = new TextField();
      textPixel.setSize(new java.awt.Dimension(140, 23));
      textPixel.setColumns(12);
      textPixel.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(java.awt.event.ActionEvent e) {
          try {
            double unit = Double.parseDouble(textPixel.getText());
            apfelPanel.setUnit(unit);
            apfelPanel.repaint();
          } catch (NumberFormatException ex) {
          }
        }
      });
    }
    return textPixel;
  }

  /**
   * This method initializes panel4
   * 
   * @return java.awt.Panel
   */
  private Panel getPanel4() {
    if (panel4 == null) {
      label4 = new Label();
      label4.setText("Iterations");
      panel4 = new Panel();
      panel4.add(label4, null);
      panel4.add(getTextIterations(), null);
    }
    return panel4;
  }

  /**
   * This method initializes textField
   * 
   * @return java.awt.TextField
   */
  private TextField getTextIterations() {
    if (textIterations == null) {
      textIterations = new TextField();
      textIterations.setColumns(10);
      textIterations.setSize(new java.awt.Dimension(120, 23));
      textIterations.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(java.awt.event.ActionEvent e) {
          try {
            int iter = Integer.parseInt(textIterations.getText());
            apfelPanel.setIterations(iter);
            apfelPanel.repaint();
          } catch (NumberFormatException ex) {
          }
        }
      });
    }
    return textIterations;
  }

  /**
   * This method initializes buttonReset
   * 
   * @return java.awt.Button
   */
  private Button getButtonReset() {
    if (buttonReset == null) {
      buttonReset = new Button();
      buttonReset.setLabel("Reset");
      buttonReset.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(java.awt.event.ActionEvent e) {
          apfelPanel.setUnit(0.01D);
          apfelPanel.setX0(-2D);
          apfelPanel.setY0(-2D);
          apfelPanel.setIterations(50);
          apfelPanel.setZoomFactor(1L);
          apfelPanel.repaint();
        }
      });
    }
    return buttonReset;
  }

  /**
   * This method initializes panel5
   * 
   * @return java.awt.Panel
   */
  private Panel getPanel5() {
    if (panel5 == null) {
      panel5 = new Panel();
      panel5.add(getButtonReset(), null);
      panel5.add(getButtonRefresh(), null);
    }
    return panel5;
  }

  public void onX0Changed() {
    textX.setText(String.valueOf(apfelPanel.getX0()));
  }

  public void onY0Changed() {
    textY.setText(String.valueOf(apfelPanel.getY0()));
  }

  public void onZoomChanged() {
    textZoom.setText(String.valueOf(apfelPanel.getZoomFactor()));
  }

  public void onIterationsChanged() {
    textIterations.setText(String.valueOf(apfelPanel.getIterations()));
  }

  public void onUnitChanged() {
    textPixel.setText(String.valueOf(apfelPanel.getUnit()));
  }

  public void onTimeChanged() {
    textTime.setText(String.valueOf(apfelPanel.getTime()));
  }

  /**
   * This method initializes panel6
   * 
   * @return java.awt.Panel
   */
  private Panel getPanel6() {
    if (panel6 == null) {
      label5 = new Label();
      label5.setText("Calc. time (ms)");
      panel6 = new Panel();
      panel6.add(label5, null);
      panel6.add(getTextTime(), null);
    }
    return panel6;
  }

  /**
   * This method initializes panel7
   * 
   * @return java.awt.Panel
   */
  private Panel getPanel7() {
    if (panel7 == null) {
      panel7 = new Panel();
      panel7.add(getChoiceCalc(), null);
    }
    return panel7;
  }

  /**
   * This method initializes textField
   * 
   * @return java.awt.TextField
   */
  private TextField getTextTime() {
    if (textTime == null) {
      textTime = new TextField();
      textTime.setColumns(6);
      textTime.setEditable(false);
    }
    return textTime;
  }

  /**
   * This method initializes choiceCalc
   * 
   * @return java.awt.Choice
   */
  private Choice getChoiceCalc() {
    if (choiceCalc == null) {
      choiceCalc = new Choice();
      choiceCalc.setSize(100,30);
      choiceCalc.addItemListener(new java.awt.event.ItemListener() {
        public void itemStateChanged(java.awt.event.ItemEvent e) {
          onCalculationChanged((String) e.getItem());
        }
      });

      String[] modes = factory.getModes();
      for (int i = 0; i < modes.length; i++)
        choiceCalc.add(modes[i]);

      onCalculationChanged(modes[0]);
    }
    return choiceCalc;
  }

  protected void onCalculationChanged(String mode) {
    try {
      ApfelCalculator calc = factory.create(mode);
      apfelPanel.setCalculator(calc);
    } catch (ApfelCalculatorException e) {
      apfelPanel.setCalculator(null);
      e.printStackTrace();
    }
    apfelPanel.repaint();
  }

  /**
   * This method initializes buttonRefresh
   * 
   * @return java.awt.Button
   */
  private Button getButtonRefresh() {
    if (buttonRefresh == null) {
      buttonRefresh = new Button();
      buttonRefresh.setLabel("Refresh");
      buttonRefresh.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(java.awt.event.ActionEvent e) {
          apfelPanel.refresh();
        }
      });
    }
    return buttonRefresh;
  }

  /**
   * @param args
   */
  public static void main(String[] args) {
    try {
      ORB orb = ORB.init(args, null);

      Frame frame = new ApfelFrame(orb);
      frame.show();
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
} // @jve:decl-index=0:visual-constraint="14,-20"
