package apfel;

import java.awt.Color;
import java.awt.Event;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.LayoutManager;
import java.awt.Panel;
import java.util.Date;

import apfel.calc.ApfelCalculator;
import apfel.calc.ApfelCalculatorException;

public class ApfelPanel extends Panel {

  private static final long serialVersionUID = 1L;

  ApfelFrame                parent           = null;

  ApfelCalculator           calculator       = null;

  Color                     mandelbrot;

  Color                     background;

  int[]                     red;

  int[]                     green;

  int[]                     blue;

  double                    unit;

  double                    x0;

  double                    y0;

  int                       iterations;

  long                      zoomFactor;

  long                      time             = 0;

  Image                     buffer;

  public ApfelPanel() {
    super();

    initialize();
  }

  public ApfelPanel(LayoutManager arg0) {
    super(arg0);

    initialize();
  }

  /**
   * This method initializes this
   * 
   * @return void
   */
  private void initialize() {

    mandelbrot = new Color(170, 0, 0);
    background = new Color(255, 255, 255);
    red = new int[] { 255, 125, 2 };
    green = new int[] { 255, 25, 7 };
    blue = new int[] { 255, 85, 3 };

    unit = 0.01D;
    x0 = -2D;
    y0 = -2D;
    iterations = 50;
    zoomFactor = 1L;

    setBackground(background);
  }

  public void paint(Graphics g) {
    if (buffer == null)
      drawApple();

    g.drawImage(buffer, 0, 0, this);
  }

  public void drawApple() {
    int l_width = 400, l_heigth = 400;
    buffer = createImage(l_width, l_heigth);
    Graphics gBuffer = buffer.getGraphics();

    if (null == calculator) {
      gBuffer.drawString("Calculator object not found", 5, 20);
    } else
      try {
        Date end, start = new Date();
        int[][] arrIterations = calculator.calcMandelbrot(x0, unit, l_width,
            y0, unit, l_heigth, iterations);
        end = new Date();
        setTime(end.getTime() - start.getTime());

        for (int y = 0; y < l_heigth; y++) {
          for (int x = 0; x < l_width; x++) {
            int realIterations = arrIterations[x][y];
            if (realIterations == iterations) {
              gBuffer.setColor(mandelbrot);
            } else {
              Color colPeriphery = new Color(red[0] - (realIterations % red[1])
                  * red[2], green[0] - (realIterations % green[1]) * green[2],
                  blue[0] - (realIterations % blue[1]) * blue[2]);
              gBuffer.setColor(colPeriphery);
            }
            gBuffer.drawLine(x, y, x, y);
          }
        }
      } catch (ApfelCalculatorException e) {
        gBuffer.drawString("Error during calculation!\n" + e.getMessage(), 5,
            20);
        e.printStackTrace();
      }
  }

  public boolean mouseDown(Event ev, int xDown, int yDown) {
    buffer = null;
    if (yDown < 400) {
      double unit_old = unit;
      if (!ev.metaDown()) {
        if (iterations == 0x3b3738)
          return true;
        setUnit(unit * 0.5D);
        setIterations((int) ((double) iterations * 1.3D));
        setZoomFactor(zoomFactor * 2L);
      } else if (ev.metaDown()) {
        if (zoomFactor < 2L)
          return true;
        setUnit(unit * 2D);
        setIterations((int) ((double) iterations / 1.3D + 0.5D));
        setZoomFactor((int) (zoomFactor * 0.5D));
      }
      setX0((x0 - (double) (200 - xDown) * unit) + (unit_old - unit)
          * (double) xDown);
      setY0((y0 - (double) (200 - yDown) * unit) + (unit_old - unit)
          * (double) yDown);
      repaint();
    }
    return true;
  }

  public void refresh() {
    buffer = null;
    repaint();
  }

  public double getX0() {
    return x0;
  }

  public void setX0(double x0) {
    buffer = null;
    this.x0 = x0;
    parent.onX0Changed();
  }

  public double getY0() {
    return y0;
  }

  public void setY0(double y0) {
    buffer = null;
    this.y0 = y0;
    parent.onY0Changed();
  }

  public long getZoomFactor() {
    return zoomFactor;
  }

  public void setZoomFactor(long zoomFactor) {
    buffer = null;
    this.zoomFactor = zoomFactor;
    parent.onZoomChanged();
  }

  public int getIterations() {
    return iterations;
  }

  public void setIterations(int iterations) {
    buffer = null;
    this.iterations = iterations;
    parent.onIterationsChanged();
  }

  public double getUnit() {
    return unit;
  }

  public void setUnit(double unit) {
    buffer = null;
    this.unit = unit;
    parent.onUnitChanged();
  }

  public void setParent(ApfelFrame parent) {
    this.parent = parent;
  }

  public long getTime() {
    return time;
  }

  protected void setTime(long time) {
    this.time = time;
    parent.onTimeChanged();
  }

  public ApfelCalculator getCalculator() {
    return calculator;
  }

  public void setCalculator(ApfelCalculator calculator) {
    buffer = null;
    this.calculator = calculator;
  }
}
